/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/dct/idct_int_aan.hh,v 1.4 2002/05/09 22:00:31 farin Exp $

    Accurate floating-point implementation of 8x8 DCT.
 ********************************************************************************
    Copyright (C) 2002 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_DCT_IDCT_INT_AAN_HH
#define LIBVIDEOCODING_DCT_IDCT_INT_AAN_HH

#include <libvideocoding/dct/dct.hh>


namespace videocoding {

  template <class OutputType> class IDCTBlk_Integer : public IDCTBlk8x8<OutputType>
  {
  public:
    IDCTBlk_Integer(int cliplow,int cliphigh);

    void CalcIDCT(OutputType* out,int out_stride,const DCTBlock_8x8&,bool add);
    const char* Name() const { return "scalar integer (AAN)"; }

  private:
    int d_cliplow,d_cliphigh;
  };


  // ----------------------------- implementation --------------------------------

  /* D.Farin: I've adapted the interface to match our API */

  void idct(int16 *block);
  void init_idct();

  template <class OutputType> IDCTBlk_Integer<OutputType>::IDCTBlk_Integer(int cliplow,int cliphigh)
  {
    d_cliplow  = cliplow;
    d_cliphigh = cliphigh;

    init_idct();
  }


  template <class OutputType> 
  void IDCTBlk_Integer<OutputType>::CalcIDCT(OutputType* out,int out_stride,
					     const DCTBlock_8x8& blk,bool add)
  {
    int16 coeff[8*8];
    for (int u=0;u<8;u++)
      for (int v=0;v<8;v++)
	coeff[8*u+v]=blk.coeff[u][v];

    idct(coeff);

    if (add)
      for (int y=0;y<8;y++)
	for (int x=0;x<8;x++)
	  {
	    int sum = out[y*out_stride+x] + coeff[y*8+x];
	    if (sum<d_cliplow)  sum=d_cliplow;
	    if (sum>d_cliphigh) sum=d_cliphigh;
	    out[y*out_stride+x] = sum;
	  }
    else
      for (int y=0;y<8;y++)
	for (int x=0;x<8;x++)
	  {
	    if (coeff[y*8+x]<d_cliplow)  coeff[y*8+x]=d_cliplow;
	    if (coeff[y*8+x]>d_cliphigh) coeff[y*8+x]=d_cliphigh;
	    out[y*out_stride+x] = coeff[y*8+x];
	  }
  }

}

#endif

