/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "config.h"
#include "libvideocoding/mpeg2/encoder/quantize.hh"
#include "libvideocoding/mpeg2/encoder/quantize_scalar.hh"
#if ENABLE_MMX
#include "libvideocoding/mpeg2/encoder/quantize_mmx.hh"
#endif
#include <libvideogfx/arch/cpu.hh>

#include <iostream>

namespace videocoding {
  using namespace std;


  void MP2_Quantizer::QuantizeMB(MP2_Macroblock& out,const MP2_Macroblock& in,int32& out_of_range,int16& cbp)
  {
    cbp = 0;

    int16 cbpbit = 0x800;

    int qscale = AskQScaleTab(out.slicehdr->pichdr->q_scale_type)[out.qcode];
    int nblks = AskNBlocks(out.slicehdr->pichdr->seqhdr->chroma_format);

    if (in.motion.prediction_mode == MP2_Intra)
      {
	for (int i=0;i<nblks;i++)
	  {
	    if (Quantize_Intra(out.AskBlock(i), in.AskBlock(i),qscale,out_of_range))
	      cbp |= cbpbit;

	    cbpbit >>= 1;
	  }
      }
    else
      {
	for (int i=0;i<nblks;i++)
	  {
	    if (Quantize_Inter(out.AskBlock(i), in.AskBlock(i),qscale,out_of_range))
	      cbp |= cbpbit;

	    cbpbit >>= 1;
	  }
      }
  }


  void MP2_Quantizer::DequantizeMB(MP2_Macroblock& out,const MP2_Macroblock& in)
  {
    int qscale = AskQScaleTab(in.slicehdr->pichdr->q_scale_type)[in.qcode];
    int nblks = AskNBlocks(in.slicehdr->pichdr->seqhdr->chroma_format);

    if (in.motion.prediction_mode == MP2_Intra)
      {
	for (int i=0;i<nblks;i++)
	  Dequantize_Intra(out.AskBlock(i), in.AskBlock(i),qscale);
      }
    else
      {
	for (int i=0;i<nblks;i++)
	  Dequantize_Inter(out.AskBlock(i), in.AskBlock(i),qscale);
      }
  }


  MP2_Quantizer* GetQuantizer(bool accurate)
  {
    CPU_Capabilities* cpu = CPU_Capabilities::AskCapabilities();

#if ENABLE_MMX
    if (cpu->AskArchitecture()==CPUArch_X86 && cpu->HasMMX())
      {
	return new MP2_Quantizer_MMX;
      }
#endif

    return new MP2_Quantizer_Scalar;
  }

}
