/********************************************************************************
  mpeg2/syntax/errors.hh
    Typical MPEG bit-stream errors.

  purpose:

  notes:

  to do:

  author(s):
   - Dirk Farin, dirk.farin@gmx.de

  modifications:
   03/Mar/2002 - Dirk Farin
     - move code to independent file
 ********************************************************************************
    Copyright (C) 1999  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_SYNTAX_ERRORS_HH
#define LIBVIDEOCODING_MPEG2_SYNTAX_ERRORS_HH

#include <libvideogfx/types.hh>
#include <libvideogfx/error.hh>

namespace videocoding {
  using namespace videogfx;

  // base-class; do not create errors of this kind
  class Excpt_BitstreamError : public Excpt_Text
  {
  public:
    Excpt_BitstreamError(const char* txt)
      : Excpt_Text(ErrSev_Error,txt)
    { }
  };

  /* Bit-stream contains erroneous data that cannot be decoded.
   */
  class Excpt_InvalidData : public Excpt_BitstreamError
  {
  public:
    Excpt_InvalidData(const char* descr)
      : Excpt_BitstreamError("invalid input data: ")
    {
      AppendText(descr);
    }
  };


  /* Data ended unexpectedly.
   */
  class Excpt_UnexpectedEndOfData : public Excpt_BitstreamError
  {
  public:
    Excpt_UnexpectedEndOfData(const char* descr = "(unknown)")
      : Excpt_BitstreamError("unexpected end of data in ")
    {
      AppendText(descr);
    }
  };

}

#endif
