/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/mpeg2/syntax/video.hh,v 1.6 2002/05/20 21:36:28 farin Exp $

    Bit-stream header structures as defined in the MPEG-1/2 standard.

  TODO
    - Correct storing of non-default chroma quantization matrices.
 ********************************************************************************
    Copyright (C) 2002 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_SYNTAX_VIDEO_HH
#define LIBVIDEOCODING_MPEG2_SYNTAX_VIDEO_HH

#include <libvideogfx/types.hh>
#include <libvideocoding/datatypes/dct.hh>
#include <libvideocoding/mpeg2/syntax/system.hh>
#include <libvideogfx/utility/bitstream/bitreader.hh>
#include <libvideogfx/utility/bitstream/bitbuffer.hh>
#include <iostream>

namespace videocoding {
  using namespace videogfx;


  // ---------- aspect_ratio_information ------------

  enum MP2_AspectRatio
    {
      MP2_AspectRatio_SAR_Square = 0x01,     // Sample Aspect Ratio = 1.0  (square pixels)
      MP2_AspectRatio_DAR_3_4    = 0x02,     // Display Aspect Ratio = 3/4
      MP2_AspectRatio_DAR_9_16   = 0x03,     // Display Aspect Ratio = 9/16
      MP2_AspectRatio_DAR_2_2_21 = 0x04      // Display Aspect Ratio = 2/2.21
    };


  // ---------- frame_rate_code ------------

  enum MP2_FrameRate
    {
      MP2_FrameRate_23_976 = 0x01,  // 23.976 fps
      MP2_FrameRate_24     = 0x02,  // 24     fps
      MP2_FrameRate_25     = 0x03,  // 25     fps
      MP2_FrameRate_29_97  = 0x04,  // 29.97  fps
      MP2_FrameRate_30     = 0x05,  // 30     fps
      MP2_FrameRate_50     = 0x06,  // 50     fps
      MP2_FrameRate_59_94  = 0x07,  // 59.94  fps
      MP2_FrameRate_60     = 0x08   // 60     fps
    };


  // --------- profile_identification ----------

  enum MP2_ProfileID
    {
      MP2_Profile_None        = 0,

      MP2_Profile_Simple      = 0x5,
      MP2_Profile_Main        = 0x4,
      MP2_Profile_SNRScalable = 0x3,
      MP2_Profile_SpatiallyScalable = 0x2,
      MP2_Profile_High        = 0x1,

      MP2_Profile_MPEG1_Constrained = 0x7
    };


  // --------- level_identification ------------

  enum MP2_LevelID
    {
      MP2_Level_None     = 0,

      MP2_Level_Low      = 0xA,
      MP2_Level_Main     = 0x8,
      MP2_Level_High1440 = 0x6,
      MP2_Level_High     = 0x4,

      MP2_Level_MPEG1_Constrained = 0xF
    };


  // --------- chroma_format ----------

  enum MP2_ChromaFormat
    {
      MP2_Chroma_420 = 1,
      MP2_Chroma_422 = 2,
      MP2_Chroma_444 = 3
    };

  inline int AskNBlocks(MP2_ChromaFormat c)
  {
    if (c==MP2_Chroma_420) return 6;
    else if (c==MP2_Chroma_422) return 8;
    else return 12;
  }


  // --------- picture_coding_type --------

  enum MP2_PictureCodingType
    {
      MP2_PicType_I = 1,
      MP2_PicType_P = 2,
      MP2_PicType_B = 3,
      MP2_PicType_D = 4
    };


  // -------- picture_structure ---------

  enum MP2_PictureStructure
    {
      MP2_PicStruct_TopField     = 1,
      MP2_PicStruct_BottomField  = 2,
      MP2_PicStruct_FramePicture = 3
    };


  // -------- prediction modes ----------

  enum MP2_Prediction_Mode
    {
      MP2_Intra = 0x00,

      MP2_Frame_FieldBased = 0x01,
      MP2_Frame_FrameBased = 0x02,
      MP2_Frame_DualPrime  = 0x03,

      MP2_Field_FieldBased = 0x11,
      MP2_Field_16x8MC     = 0x12,
      MP2_Field_DualPrime  = 0x13
    };





  struct MP2_SequenceHeader
  {
    MP2_SequenceHeader() { }
    MP2_SequenceHeader(bool mpeg2);

    void SetDefaults(bool mpeg2);

    uint14              width,height;
    MP2_AspectRatio     aspect_ratio_information;
    uint16              framerate100;
    uint30              bit_rate;
    uint18              vbv_buffer_size;
    bool                constrained_parameters_flag;
  			
    QMatrix_8x8         intra_matrix;
    QMatrix_8x8         chroma_intra_matrix;
  			
    QMatrix_8x8         nonintra_matrix;
    QMatrix_8x8         chroma_nonintra_matrix;
  
    // MPEG2

    bool                IsMPEG2;
  			
    MP2_ProfileID       profile;
    MP2_LevelID         level;
    bool                progressive_sequence;
    MP2_ChromaFormat    chroma_format;
    bool                low_delay;


    int  AskWidthInMBs() const { return (width+15)/16; }
    int  AskHeightInMBs() const { return (height+15)/16; }
    int  AskTotalNrOfMBs() const { return AskWidthInMBs()*AskHeightInMBs(); }


    void Read(class BitReader&);
    void Write(class BitBuffer&) const;

    bool HasNonDefaultIntraQMatrix() const;
    bool HasNonDefaultInterQMatrix() const;
  };


  struct MP2_GOPHeader
  {
    MP2_GOPHeader();
  
    void AdvanceOnePicture(int fps);
  
    bool  time_code_drop_frame_flag;
    uint5 time_code_hours;
    uint6 time_code_minutes;
    uint6 time_code_seconds;
    uint6 time_code_pictures;
    bool  closed_gop;
    bool  broken_link;

    void Read(class BitReader&);
    void Write(class BitBuffer&) const;
  };


  struct MP2_PictureHeader
  {
    MP2_PictureHeader();

    MP2_SequenceHeader* seqhdr;

    uint10                  temporal_reference;
    MP2_PictureCodingType   picture_coding_type;
    uint16                  vbv_delay;
    bool                    full_pel_vector[2];
#define full_pel_forward_vector  full_pel_vector[0]
#define full_pel_backward_vector full_pel_vector[1]
#define forward_f_code  f_code[0][0]
#define backward_f_code f_code[1][0]
  
    // MPEG2

    bool                 has_coding_extension;  /* This must have the same value as
						     m_IsMPEG2 in the sequence header. */
  
    uint4                f_code[2 /* temporal direction 0=forw/1=back */]
    [2 /* spatial direction 0=horiz/1=vert */];
    uint8                intra_dc_precision;  /* 8-11 */
    MP2_PictureStructure picture_structure;
    bool                 top_field_first;
    bool                 frame_pred_frame_dct;
    bool                 concealment_motion_vectors;
    uint1                q_scale_type;
    uint1                intra_vlc_format;
    bool                 alternate_scan;
    bool                 repeat_first_field;
    bool                 progressive_frame;
    bool                 composite_display_flag;
    uint1                v_axis;
    uint3                field_sequence;
    bool                 sub_carrier;
    uint7                burst_amplitude;
    uint8                sub_carrier_phase;

    bool                 has_PTS;
    int64                PTS;

    void Read(class BitReader&);
    void Write(class BitBuffer&) const;
  };


  struct MP2_SliceHeader
  {
    MP2_SliceHeader() : pichdr(NULL), CodeIntraSlice(false) { }

    MP2_PictureHeader* pichdr;

    uint10 slice_vertical_position;
    uint7  priority_breakpoint;
    uint5  qcode;
    bool   CodeIntraSlice;
    bool   intra_slice;

    void Read(class BitReader&);
    void Write(class BitBuffer&) const;
  };



  std::ostream& operator<<(std::ostream&,const MP2_SequenceHeader&);
  std::ostream& operator<<(std::ostream&,const MP2_GOPHeader&);
  std::ostream& operator<<(std::ostream&,const MP2_PictureHeader&);
  std::ostream& operator<<(std::ostream&,const MP2_SliceHeader&);


  struct Scan { uint8 u,v; };
  const Scan* AskScan(bool alternate_scan);  // return array which maps 0-63 to u,v indices

  const int* AskQScaleTab(bool nonuniform);  // return array which maps qcode to qscale

  extern QMatrix_8x8 MP2_DefaultQuantMatrix_Intra;
  extern QMatrix_8x8 MP2_DefaultQuantMatrix_Inter;

}

#endif
